@@ -28,16 +28,11 @@
 
 G_BEGIN_DECLS
 
-void     gda_data_model_signal_emit_changed             (GdaDataModel *model);
-
 void     gda_data_model_row_inserted        (GdaDataModel *model, gint row);
 void     gda_data_model_row_updated         (GdaDataModel *model, gint row);
 void     gda_data_model_row_removed         (GdaDataModel *model, gint row);
 void     gda_data_model_reset               (GdaDataModel *model);
 
-gboolean gda_data_model_move_iter_at_row_default (GdaDataModel *model, GdaDataModelIter *iter, gint row);
-gboolean gda_data_model_move_iter_next_default   (GdaDataModel *model, GdaDataModelIter *iter);
-gboolean gda_data_model_move_iter_prev_default   (GdaDataModel *model, GdaDataModelIter *iter);
 G_END_DECLS
 
 #endif
