@@ -314,10 +314,12 @@ gda_data_model_reset (GdaDataModel *model)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 
-	g_signal_emit (G_OBJECT (model),
-		       gda_data_model_signals[RESET], 0);
-
-	_gda_data_model_signal_emit_changed (model);
+	if (do_notify_changes (model)) {
+		g_signal_emit (G_OBJECT (model),
+			       gda_data_model_signals[RESET], 0);
+		
+		_gda_data_model_signal_emit_changed (model);
+	}
 }
 
 /**
@@ -1960,11 +1962,12 @@ static gchar *
 real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes, 
 				    gboolean dump_rows, gboolean dump_title, gboolean null_as_empty, GError **error)
 {
+#define ERROR_STRING "####"
 #define MULTI_LINE_NO_SEPARATOR
 
 	gboolean allok = TRUE;
 	GString *string;
-	gchar *offstr, *str;
+	gchar *str;
 	gint n_cols, n_rows;
 	gint *cols_size;
 	gboolean *cols_is_num;
@@ -1983,10 +1986,6 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 
 	string = g_string_new ("");
 
-        /* string for the offset */
-        offstr = g_new0 (gchar, offset+1);
-	memset (offstr, ' ', offset);
-
 	/* compute the columns widths: using column titles... */
 	n_cols = gda_data_model_get_n_columns (model);
 	n_rows = gda_data_model_get_n_rows (model);
@@ -2043,23 +2042,24 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	for (j = 0; j < n_rows; j++) {
 		for (i = 0; i < n_cols; i++) {
 			if (! dump_attributes) {
-				value = gda_data_model_get_value_at (model, i, j, error);
+				value = gda_data_model_get_value_at (model, i, j, NULL);
 				if (!value) {
-					allok = FALSE;
-					goto out;
+					cols_size [i + col_offset] = MAX (cols_size [i + col_offset], strlen (ERROR_STRING));
 				}
-				str = NULL;
-				if (null_as_empty) {
-					if (!value || gda_value_is_null (value))
-						str = g_strdup ("");
-				}
-				if (!str)
-					str = value ? gda_value_stringify ((GValue*)value) : g_strdup ("_null_");
-				if (str) {
-					gint w;
-					string_get_dimensions (str, &w, NULL);
-					cols_size [i + col_offset] = MAX (cols_size [i + col_offset], w);
-					g_free (str);
+				else {
+					str = NULL;
+					if (null_as_empty) {
+						if (!value || gda_value_is_null (value))
+							str = g_strdup ("");
+					}
+					if (!str)
+						str = value ? gda_value_stringify ((GValue*)value) : g_strdup ("_null_");
+					if (str) {
+						gint w;
+						string_get_dimensions (str, &w, NULL);
+						cols_size [i + col_offset] = MAX (cols_size [i + col_offset], w);
+						g_free (str);
+					}
 				}
 			}
 			else {
@@ -2154,18 +2154,18 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 		
 		for (i = 0; i < n_cols; i++) {
 			if (!dump_attributes) {
-				value = gda_data_model_get_value_at (ramodel, i, j, error);
-				if (!value) {
-					allok = FALSE;
-					goto out;
+				value = gda_data_model_get_value_at (ramodel, i, j, NULL);
+				if (!value)
+					str = g_strdup (ERROR_STRING);
+				else {
+					str = NULL;
+					if (null_as_empty) {
+						if (!value || gda_value_is_null (value))
+							str = g_strdup ("");
+					}
+					if (!str)
+						str = value ? gda_value_stringify ((GValue *)value) : g_strdup ("_null_");
 				}
-				str = NULL;
-				if (null_as_empty) {
-					if (!value || gda_value_is_null (value))
-						str = g_strdup ("");
-				}
-				if (!str)
-					str = value ? gda_value_stringify ((GValue *)value) : g_strdup ("_null_");
 			}
 			else {
 				GdaValueAttribute attrs;
@@ -2259,8 +2259,6 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	g_free (cols_size);
 	g_free (cols_is_num);
 
-	g_free (offstr);
-	
 	if (allok) {
 		str = string->str;
 		g_string_free (string, FALSE);
