@@ -35,8 +35,6 @@
 #include <locale.h>
 #endif
 
-extern gchar *gda_numeric_locale;
-
 #define _GDA_PSTMT(x) ((GdaPStmt*)(x))
 
 enum
@@ -566,9 +564,6 @@ gda_mysql_recordset_fetch_nb_rows (GdaDataSelect *model)
 	if (model->advertized_nrows >= 0)
 		return model->advertized_nrows;
 
-	/* use C API to determine number of rows,if possible */
-	// TO_IMPLEMENT;
-	
 	model->advertized_nrows = mysql_stmt_affected_rows (imodel->priv->mysql_stmt);
 	
 	return model->advertized_nrows;
@@ -736,10 +731,11 @@ new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, gint rownum, GError **error)
 
 			break;
 		case MYSQL_TYPE_FLOAT:
-		case MYSQL_TYPE_DOUBLE:
+		case MYSQL_TYPE_DOUBLE: {
+			char *current_locale;
 			g_memmove (&doublevalue, mysql_bind_result[i].buffer, sizeof(double));
 			
-			setlocale (LC_NUMERIC, "C");
+			current_locale = setlocale (LC_NUMERIC, "C");
 			if (type == G_TYPE_FLOAT)
 				g_value_set_float (value, (float) doublevalue);
 			else if (type == G_TYPE_DOUBLE)
@@ -751,9 +747,10 @@ new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, gint rownum, GError **error)
 					     _("Type %s not mapped for value %f"),
 					     g_type_name (type), doublevalue);
 			}
-			setlocale (LC_NUMERIC, gda_numeric_locale);
+			setlocale (LC_NUMERIC, current_locale);
 			
 			break;
+		}
 		case MYSQL_TYPE_STRING:
 		case MYSQL_TYPE_VAR_STRING:
 		case MYSQL_TYPE_BLOB:
@@ -780,17 +777,22 @@ new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, gint rownum, GError **error)
 				gda_value_set_binary (value, &binary);
 			}
 			else if (type == GDA_TYPE_BLOB) {
-				GdaBinary binary = {
-					.data = (guchar*) strvalue,
-					.binary_length = length
+				/* web don't use GdaMysqlBlobOp because it looks like the MySQL
+				 * API does not support BLOBs accessed in a random way,
+				 * so we return the whole BLOB at once */
+				GdaBlob blob = {
+					.data.data = (guchar*) strvalue,
+					.data.binary_length = length,
+					.op = NULL
 				};
-				gda_value_set_binary (value, &binary);
+				gda_value_set_blob (value, &blob);
 			}
 			else if (type == G_TYPE_DOUBLE) {
 				if (length > 0) {
-					setlocale (LC_NUMERIC, "C");
+					char *current_locale;
+					current_locale = setlocale (LC_NUMERIC, "C");
 					g_value_set_double (value, atof (strvalue));
-					setlocale (LC_NUMERIC, gda_numeric_locale);
+					setlocale (LC_NUMERIC, current_locale);
 				}
 				else {
 					/* error: wrong column type */
